// Pausable timers manager
::timers <- {
	manager = Entities.FindByClassname(null, "worldspawn")
	latestId = 0
	getId = function() {
		return latestId++
	}
}

// Delegate table for timers
::timersDelegate <- {

    // Returns how much time should be left before calling the callback, this accounts for pauses
	getTimeLeft = function() {
		return duration - (Time() - (start + durationPaused + (pauseStart ? (Time() - pauseStart) : 0)))
	}

    // Pause the timer
	pauseTimer = function() {
		if (pauseStart) return
		segmentId++
		pauseStart = Time()
		return this
	}

    // Resume the timer
	resumeTimer = function() {
		if (!pauseStart) return
		segmentId++
		durationPaused += (Time() - pauseStart)
		pauseStart = null
		EntFireByHandle(::timers.manager, "RunScriptCode", "if ("+id+" in ::timers && ::timers["+id+"].segmentId == "+segmentId+") ::timers["+id+"].endTimer()", getTimeLeft(), null, null)
		return this
	}

    // Delete the timer and call the callback
	endTimer = function() {
		deleteTimer()
		onDone()
		return this
	}

    // Delete the timer without calling the callback
	deleteTimer = function() {
		if (id in ::timers) delete ::timers[id]
		return this
	}

	getCopy = function() {
		return ::createTimer(duration, onDone)
	}

}

// Create a new pausable timer
::createTimer <- function(duration, onDone) {

	local timer = {
		id = ::timers.getId()
		segmentId = 0

		duration = duration
		onDone = onDone

		start = Time()
		
		pauseStart = Time()
		durationPaused = 0
	}.setdelegate(::timersDelegate)

	::timers[timer.id] <- timer
	timer.resumeTimer()

	return timer
}

// Delegate table for timelines
::timelinesDelegate <- {

    // Start the timeline with the keyframes
	setup = function() {
		timers = keyframes.map(@(kf) ::createTimer(kf[0], kf[1]))
		return this
	}

    // Pause the timers that make up the timeline
	pauseTimeline = function() {
		foreach (timer in timers) timer.pauseTimer()
		return this
	}

    // Resume the timeline
	resumeTimeline = function() {
		foreach (timer in timers) timer.resumeTimer()
		return this
	}

    // Delete the timeline, thus preventing any further effects
	deleteTimeline = function() {
		foreach (timer in timers) timer.deleteTimer()
		return this
	}

    // Restart the timeline, can be used to loop the timeline if it isn't anonymous
	restartTimeline = function() {
		deleteTimeline()
		return setup()
	}

}

// Create a timeline from the given keyframes
// keyframes: [delay, callback][]
::createTimeline <- function(keyframes = []) {

	local timeline = {
		keyframes = keyframes
		timers = []
	}.setdelegate(::timelinesDelegate)

	return timeline.setup()
}