// Get an array of entities with the given name
::collect <- function(name, method = "FindByName") {
	local collection = []
	for (local ent; ent = Entities[method](ent,name);) {
		collection.push(ent)
	}
	return collection
}

// Get the first entity that matches the given name
::first <- function(name, method = "FindByName", filter = null) {
	if (!filter) return Entities[method](null, name)
	for (local ent; ent = Entities[method](ent,name);) {
		if (filter(ent)) return ent
	}
	return null
}

// Get an entity's validated scope
::getScope <- function(ent) {
	ent.ValidateScriptScope()
	return ent.GetScriptScope()
}

// Trigger a entity's input at the end of the frame
::fire <- function(target, action, value = "", delay = 0, activator = null, caller = null) {
	EntFireByHandle(target, action, ""+value, delay, activator, caller)
}

// Trigger a entity's input
::input <- function(target, action, value = "", activator = null, caller = null) {
	target.AcceptInput(action, ""+value, activator, caller)
}

// Print an error to the console
::printError <- function(message) {
	printl("┌───────┬─ ─ ─")
	printl("│ Error ♦ "+message)
	printl("└───────┴─ ─ ─")
	return null
}

// Get the team's name given the team's number
::team <- function(teamNum) {
	if (teamNum == 2) return "red"
	if (teamNum == 3) return "blue"
	return "none"
}

// Get the team's number given the team's name
::teamNum <- function(teamName) {
	if (teamName == "red") return 2
	if (teamName == "blue") return 3
	return 0
}

// Call a function in an entity's scope, if it's there
::call <- function(entity, hookName, ...) {
	local scope = ::getScope(entity)
	if (hookName in scope || ("entityDelegate" in scope && hookName in scope.entityDelegate)) {
		return {
			value = scope[hookName].acall([scope].extend(vargv))
		}
	}
	return false
}

// With this you can add an "entityDelegate" slot to any entity's scope and delegate to its table value
getroottable()._get <- function(idx) {
	if (("entityDelegate" in this) && (idx in entityDelegate)) return entityDelegate[idx]
	else throw null
}

::setEntityDelegate <- function(entity, delegate) {
	::getScope(entity).entityDelegate <- delegate
}