::roundNum <- 0
if (::roundNum++ == 0) {

	// Precaches
	PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "teleportedin_red"})
	PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "teleportedin_blue"})
	PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "electrocuted_gibbed_red"})
	PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "electrocuted_gibbed_blue"})
	PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "taunt_headbutt_impact_stars"})

	// Helpers
	IncludeScript("include_utilities")
	IncludeScript("include_array_utilities")
	IncludeScript("include_timers")
	IncludeScript("include_events")

	// Features

	// // Spawns
	IncludeScript("feature_spawns")
	::spawnpointsEntityDelegate <- {
		handleTeleportingInStart = function() {
			::fire(particleEmitter[["red", "blue"][spawn.teamNum-2]], "Start")
		}
		handleTeleportingInEnd = function() {
			::fire(particleEmitter.red, "Stop")
			::fire(particleEmitter.blue, "Stop")
		}
		handleTeleportedIn = function(player) {
			DispatchParticleEffect(spawn.teamNum == 2 ? "teleportedin_red" : "teleportedin_blue", self.GetOrigin(), Vector(0,1,0))
		}
		handleTeleportedInFailure = function() {
			::spawnDeadRobot(self.GetOrigin(), self.GetAbsAngles(), spawn.teamNum)
			DispatchParticleEffect(spawn.teamNum == 2 ? "teleportedin_red" : "teleportedin_blue", self.GetOrigin(), Vector(0,1,0))
		}
	}

	// // Capture points
	IncludeScript("feature_capture_points")

	// // Robot destruction
	IncludeScript("feature_robot_destruction_alternative")
	::robotsDelegate.handleInit = function() {
		spawnEvent.addEffect(@[this](_) findJob())
		restoreEvent.addEffect(@[this](_) ::createTimer(1, @[this]() !isSapped && findJob()))
		deathEvent.addEffect(@[this](_) ::createTimer(4, @[this]() selectSpawnpoint().spawn()))
		selectSpawnpoint()
	}
	::robotsDelegate.selectSpawnpoint <- function() {
		local selectedBotSpawnpoint = ::pickRandom(::extractPrioritized(
			::botSpawnpoints[name.slice(6)].filter(@[this](_,botSpawnpoint) botSpawnpoint != spawnpoint),
			@(botSpawnpoint) ::getScope(botSpawnpoint).priority
		))
		if (selectedBotSpawnpoint) setSpawnpoint(selectedBotSpawnpoint)
		else printError("Failed to find a spawnpoint for a robot \""+name+"\"! Make sure you have at least two bot spawnpoints per group.")
		return this
	}
	::robotsDelegate.findJob <- function(exception = null) {
		local selectedJob = ::pickRandom(::extractPrioritized(
			::jobs[name.slice(6)].filter(@(_,job) job != exception),
			@(job) ::getScope(job).priority - (::getScope(job).workers.len()*5)
		))
		if (selectedJob) goTo(selectedJob)
		else printError("Failed to find a job for a robot \""+name+"\"! Make sure you have at least two jobs per group.")
	}
	::jobsEntityDelegate <- {
		removeWorker = function(worker) {
			worker.robot.deathEvent.removeEffect(worker.onDeath)
			worker.robot.sapEvent.removeEffect(worker.onSap)
			local workerToRemoveIdx = workers.find(worker)
			workers.remove(workerToRemoveIdx)
			if (workers.len() == 0) onEmptyWorkers()
			else if (workerToRemoveIdx == 0) onNewFirstWorker()
		}
		handleRobotArrival = function(robot) {
			local worker = {}
			worker.robot <- robot
			worker.onDeath <- @[this](_) removeWorker(worker)
			worker.onSap <- @[this](_) removeWorker(worker)
			worker.robot.deathEvent.addEffect(worker.onDeath)
			worker.robot.sapEvent.addEffect(worker.onSap)
			workers.push(worker)
			if (workers.len() == 1) onNewFirstWorker()
		}
		handleRobotIncoming = function(robot) {
			priority--
			robot.doGoTo(self)
		}
		onCompletedJob = function() {
			DispatchParticleEffect("taunt_headbutt_impact_stars", self.GetOrigin()-Vector(0,0,32), Vector(0,0,0))
			if (workers[0].robot.teamNum == ::cap[workers[0].robot.name.slice(6)].teamNum) {
				local pointsAmt = NetProps.GetPropFloat(::logicRD, "m_flRobotScoreInterval")
				workers[0].robot.teamNum == 2
					? ::match.score.red.add(pointsAmt)
					: ::match.score.blue.add(pointsAmt)
			}
			workers[0].robot.findJob(self)
			removeWorker(workers[0])
		}
		stopParticles = function() {
			::fire(particleEmitter.red, "Stop")
			::fire(particleEmitter.blue, "Stop")
		}
		onEmptyWorkers = function() {
			jobTimeline.restartTimeline().pauseTimeline()
			stopParticles()
		}
		onNewFirstWorker = function() {
			jobTimeline.restartTimeline().resumeTimeline()
			local correctParticleEmitter = particleEmitter[workers[0].robot.teamNum == 2 ? "red" : "blue"]
			correctParticleEmitter.SetAbsOrigin(workers[0].robot.bot.GetOrigin()+Vector(0,0,32))
			::input(correctParticleEmitter, "SetParent", "!activator", workers[0].robot.bot)
			stopParticles()
			::fire(correctParticleEmitter, "Start")
		}
	}
	::botSpawnpointsEntityDelegate <- {
		handleRobotTeleport = function(robot) {
			priority--
			::createTimeline([
				[0, @[this]() DispatchParticleEffect(robot.teamNum == 2 ? "electrocuted_gibbed_red" : "electrocuted_gibbed_blue", self.GetOrigin(), Vector(0,0,0))],
				[0, @[this]() DispatchParticleEffect(robot.teamNum == 2 ? "electrocuted_gibbed_red" : "electrocuted_gibbed_blue", self.GetOrigin(), Vector(0,0,0))],
				[2, @[this]() DispatchParticleEffect(robot.teamNum == 2 ? "teleportedin_red" : "teleportedin_blue", self.GetOrigin(), Vector(0,1,0))],
				[2, @[this]() robot.doTeleportTo(self)]
			])
		}
	}

}

// Setup the spawning
::setupSpawning()

// Setup the robots objectives
::jobsArray <- ::collect("job_*")
::jobs <- ::groupBy(
	::jobsArray,
	@(job) job.GetName().slice(4)
)
foreach (job in ::jobsArray) {
	local jobScope = ::getScope(job)
	jobScope.entityDelegate <- ::jobsEntityDelegate
	jobScope.workers <- []
	jobScope.priority <- 99999
	jobScope.particleEmitterTarget <- SpawnEntityFromTable("info_particle_system", {effect_name = "taunt_headbutt_impact_stars", targetname = "cpoint1_"+job.entindex(), origin = job.GetOrigin()-Vector(0,0,64)})
	jobScope.particleEmitter <- {
		red = SpawnEntityFromTable("info_particle_system", {effect_name = "medicgun_beam_red_invunglow", angles = "-90 0 0", cpoint1 = jobScope.particleEmitterTarget.GetName()})
		blue = SpawnEntityFromTable("info_particle_system", {effect_name = "medicgun_beam_blue_invunglow", angles = "-90 1 0", cpoint1 = jobScope.particleEmitterTarget.GetName()})
	}
	jobScope.jobTimeline <- ::createTimeline([
		[5, @[jobScope]() onCompletedJob()],
	]).pauseTimeline()
}

// Setup the robots spawnpoints
::botSpawnpointsArray <- ::collect("bot_spawnpoint_*")
::botSpawnpoints <- ::groupBy(
	::botSpawnpointsArray,
	@(botSpawnpoint) botSpawnpoint.GetName().slice(15)
)
foreach (botSpawnpoint in ::botSpawnpointsArray) {
	local botSpawnpointScope = ::getScope(botSpawnpoint)
	botSpawnpointScope.priority <- 99999
	botSpawnpointScope.entityDelegate <- ::botSpawnpointsEntityDelegate
}

// Trigger the RD setup
::setupRD()

// Setup the control points by connecting them to a spawn and the scoring system
foreach (controlPoint in ::collect("cap_*")) {
	local group = controlPoint.GetName().slice(4)
	local spawn = ::setupSpawn(group)
	local cap = ::setupCap(group)
	cap.handleTeamChange <- function(newTeamNum, oldTeamNum) {
		spawn.setTeam(newTeamNum).respawnPlayersWithinBoundaries().relocateSpawningPlayers()
		if (oldTeamNum == 2) ::match.passiveAdder.red.addLevel(-1)
		else if (oldTeamNum == 3) ::match.passiveAdder.blue.addLevel(-1)
		if (newTeamNum == 2) ::match.passiveAdder.red.addLevel(1)
		else if (newTeamNum == 3) ::match.passiveAdder.blue.addLevel(1)
	}
}

// Setup the teams' spawns
::setupSpawn("red", 2)
::setupSpawn("blue", 3)

// Setup the spawnpoints
foreach (sp in ::collect("spawnpoint_*")) {
	local scope = ::getScope(sp)
	scope.particleEmitter <- {
		red = SpawnEntityFromTable("info_particle_system", {effect_name = "medic_radiusheal_red_spiral", origin = sp.GetOrigin()})
		blue = SpawnEntityFromTable("info_particle_system", {effect_name = "medic_radiusheal_blue_spiral", origin = sp.GetOrigin()})
	}
	scope.entityDelegate <- ::spawnpointsEntityDelegate
}